﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public class PloyTen : RegressionToyBenchmark {

    public PloyTen() {
      Name = "Ploy-10 y = X1*X2 + X3*X4 + X5*X6 + X1*X7*X9 + X3*X6*X10";
      Description = "Paper: A Simple but Theoretically-motivated Method to Control Bloat in Genetic Programming" + Environment.NewLine
        + "Authors: Riccardo Poli" + Environment.NewLine
        + "Function: y = X1*X2 + X3*X4 + X5*X6 + X1*X7*X9 + X3*X6*X10" + Environment.NewLine
        + "Terminal set: x1, x2, x3, x4, x5, x6, x7, x8, x9, x10" + Environment.NewLine
        + "Fitness was minus the sum of the absolute values of the errors made over 50 fitness cases. "
        + "These were generated by randomly assigning values to the variables xiin the range [1, 1].";
      targetVariable = "Y";
      inputVariables = new List<string>() { "X1", "X2", "X3", "X4", "X5", "X6", "X7", "X8", "X9", "X10" };
      trainingPartition = new IntRange(0, 250);
      testPartition = new IntRange(250, 500);
    }

    protected override List<double> GenerateTarget(List<List<double>> data) {
      double x1, x2, x3, x4, x5, x6, x7, x8, x9, x10;
      List<double> results = new List<double>();
      for (int i = 0; i < data[0].Count; i++) {
        x1 = data[0][i];
        x2 = data[1][i];
        x3 = data[2][i];
        x4 = data[3][i];
        x5 = data[4][i];
        x6 = data[5][i];
        x7 = data[6][i];
        x8 = data[7][i];
        x9 = data[8][i];
        x10 = data[9][i];
        results.Add(x1 * x2 + x3 * x4 + x5 * x6 + x1 * x7 * x9 + x3 * x6 * x10);
      }
      return results;
    }

    protected override List<List<double>> GenerateInput() {
      List<List<double>> dataList = new List<List<double>>();
      DoubleRange range = new DoubleRange(-1, 1);
      for (int i = 0; i < inputVariables.Count; i++) {
        dataList.Add(RegressionBenchmark.GenerateUniformDistributedValues(testPartition.End, range));
      }
      return dataList;
    }
  }
}
