﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public class SpatialCoevolution : RegressionToyBenchmark {

    public SpatialCoevolution() {
      Name = "Spatial co-evolution F(x,y) = 1/(1+power(x,-4)) + 1/(1+pow(y,-4))";
      Description = "Paper: Evolutionary consequences of coevolving targets" + Environment.NewLine
        + "Authors: Ludo Pagie and Paulien Hogeweg" + Environment.NewLine
        + "Function: F(x,y) = 1/(1+power(x,-4)) + 1/(1+pow(y,-4))" + Environment.NewLine
        + "Terminal set: x, y" + Environment.NewLine
        + "The fitness of a solution is defined as the mean of the absolute differences between "
        + "the target function and the solution over all problems on the basis of which it is evaluated. "
        + "A solution is considered completely ’correct’ if, for all 676 problems in the ’complete’ "
        + "problem set used in the static evaluation scheme, the absolute difference between "
        + "solution and target function is less than 0:01 (this is a so-called hit)." + Environment.NewLine;
      targetVariable = "F";
      inputVariables = new List<string>() { "X", "Y" };
      trainingPartition = new IntRange(0, 338);
      testPartition = new IntRange(338, 676);
    }

    protected override List<double> GenerateTarget(List<List<double>> data) {
      double x, y;
      List<double> results = new List<double>();
      for (int i = 0; i < data[0].Count; i++) {
        x = data[0][i];
        y = data[1][i];
        results.Add(1 / (1 + Math.Pow(x, -4)) + 1 / (1 + Math.Pow(y, -4)));
      }
      return results;
    }

    protected override List<List<double>> GenerateInput() {
      List<List<double>> dataList = new List<List<double>>();

      List<double> oneVariableTestData = RegressionBenchmark.GenerateSteps(new DoubleRange(-5, 5), 0.4);
      List<List<double>> testData = new List<List<double>>() { oneVariableTestData, oneVariableTestData };
      dataList = RegressionBenchmark.GenerateAllCombinationsOfValuesInLists(testData);

      return dataList;
    }
  }
}
